from typing import Any

class Memory:
    def __init__(self) -> None: ...
    def exists(self, key) -> bool: ...
    def save(self, key, data) -> bool: ...
    def load(self, key) -> Any: ...
    def delete(self, key) -> bool: ...
