import typing as tp
from pathlib import Path

class Drive:
    def __init__(self, root: tp.Union[Path, str], create: bool = ..., unstable: bool = ...) -> None: ...
    def exists(self, key: str) -> bool: ...
    def save(self, key: str, data: bytes) -> bool: ...
    def load(self, key: str) -> bytes: ...
    def delete(self, key: str) -> bool: ...

class DriveTemp(Drive):
    def __init__(self) -> None: ...
    def __del__(self) -> None: ...
