from typing import Any, Iterator

def iterable_hexdigest(algorithm, iterator: Iterator[bytes]) -> str: ...
def hexdigest(algorithm, data: bytes, block_size=...): ...
def get_hexdigest(algorithm): ...

registry: Any

def get_hash_fn(algorithm): ...
