from typing import Any, Union

class Packaging:
    def forward(self, i) -> Any: ...
    def backward(self, o) -> Any: ...

class Conveyor:
    def __init__(self, *packer: Packaging) -> None: ...
    def forward(self, i) -> Any: ...
    def backward(self, o) -> Any: ...

class Compression:
    def __init__(self, level: int = ...) -> None: ...
    def forward(self, i: bytes) -> bytes: ...
    def backward(self, o: bytes) -> bytes: ...

class Pickling:
    def forward(self, i) -> bytes: ...
    def backward(self, o: bytes) -> Any: ...

class Base64:
    def forward(self, i: bytes) -> bytes: ...
    def backward(self, o: bytes) -> bytes: ...

class Json:
    def forward(self, i) -> str: ...
    def backward(self, o: str) -> Any: ...

class Encoding:
    def __init__(self, encoding: str = ...) -> None: ...
    def forward(self, i: str) -> bytes: ...
    def backward(self, o: bytes) -> str: ...

class Encryption:
    def __init__(self, *, key: Union[str, bytes] = ..., password: Union[str, bytes] = ..., seed: int = ...) -> None: ...
    @property
    def key(self) -> str: ...
    @staticmethod
    def generate_key(password: Union[str, bytes] = ..., seed: int = ...) -> str: ...
    def forward(self, i: bytes) -> bytes: ...
    def backward(self, o: bytes) -> bytes: ...
