class Sqlite:
    def __init__(self, path, create: bool = ...) -> None: ...
    def __del__(self) -> None: ...
    def exists(self, key: str) -> bool: ...
    def save(self, key: str, data) -> bool: ...
    def load(self, key: str): ...
    def delete(self, key: str) -> bool: ...

class SqliteTemp(Sqlite):
    def __init__(self) -> None: ...
    def __del__(self) -> None: ...
